@@ -1,16 +1,13 @@
 package Net::DNS;
 
 #
-# $Id: DNS.pm 1522 2016-12-23 11:31:34Z willem $
+# $Id: DNS.pm 1529 2017-01-18 21:46:01Z willem $
 #
-BEGIN {
-	require 5.004_04;
-	require vars;
-	'vars'->import(qw($VERSION $SVNVERSION));
-	$VERSION    = '1.06_06';
-	$VERSION    = eval $VERSION;
-	$SVNVERSION = (qw$LastChangedRevision: 1522 $)[1];
-}
+require 5.006;
+our $VERSION;
+$VERSION = '1.07_01';
+$VERSION = eval $VERSION;
+our $SVNVERSION = (qw$LastChangedRevision: 1529 $)[1];
 
 
 =head1 NAME
@@ -35,11 +32,11 @@ details.
 
 
 use strict;
+use warnings;
 use integer;
 
 use base qw(Exporter);
-use vars qw(@EXPORT);
-@EXPORT = qw(SEQUENTIAL UNIXTIME YYYYMMDDxx
+our @EXPORT = qw(SEQUENTIAL UNIXTIME YYYYMMDDxx
 		yxrrset nxrrset yxdomain nxdomain rr_add rr_del
 		mx rr rrsort);
 
