@@ -1,10 +1,9 @@
 package Net::DNS::RR;
 
 #
-# $Id: RR.pm 1513 2016-11-12 03:20:58Z willem $
+# $Id: RR.pm 1527 2017-01-18 21:42:48Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1513 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
 
 
 =head1 NAME
@@ -33,19 +32,16 @@ See also the manual pages for each specific RR type.
 
 
 use strict;
+use warnings;
 use integer;
 use Carp;
-use FileHandle;
+
+use constant LIB => grep !ref($_), @INC;
 
 use Net::DNS::Parameters;
 use Net::DNS::Domain;
 use Net::DNS::DomainName;
 
-use constant DNSEXTLANG => defined eval {
-	local $SIG{__WARN__} = sub { };
-	new FileHandle("RRTYPEgen |");
-};
-
 
 =head1 METHODS
 
@@ -237,8 +233,7 @@ sub decode {
 
 	my $index = $fixed + RRFIXEDSZ;
 	die 'corrupt wire-format data' if length $$data < $index;
-	my $type = unpack "\@$fixed n", $$data;
-	my $self = $base->_subclass("TYPE$type");
+	my $self = $base->_subclass( unpack "\@$fixed n", $$data );
 	$self->{owner} = $owner;
 	@{$self}{qw(class ttl rdlength)} = unpack "\@$fixed x2 n N n", $$data;
 
@@ -344,15 +339,12 @@ sub string {
 	my $tab = length($name) < 72 ? "\t" : ' ';
 	return join $tab, @core, '; no data' unless scalar @rdata;
 
-	my @note = map "; $_", $self->_annotation;
 	my @line = _wrap( join( $tab, @core, '(' ), @rdata, ')' );
-	return join "\n\t", @line, @note if scalar(@line) > 1;	# multi-line RR
 
-	for (@line) {						# single line RR
-		s/\s+[(]\s*/\t/;				# strip redundant ( )
-		s/\s+[)]$//;
-		return join "\n\t", $_, @note;
-	}
+	my $last = pop(@line);					# last or only line
+	$last = join $tab, @core, "@rdata" unless scalar(@line);
+
+	return join "\n\t", @line, _wrap( $last, map "; $_", $self->_annotation );
 }
 
 
@@ -629,7 +621,7 @@ The above example is the sorting function implemented in MX.
 
 =cut
 
-use vars qw(%rrsortfunct);
+our %rrsortfunct;
 
 sub set_rrsort_func {
 	my $class     = shift;
@@ -675,10 +667,8 @@ sub get_rrsort_func {
 # The optional second argument indicates that default values are
 # to be copied into the newly created object.
 
-use vars qw(%_LOADED %_MINIMAL);
-
-$_MINIMAL{ANY} = bless ['type' => 255], __PACKAGE__;
-%_LOADED = %_MINIMAL;
+our %_MINIMAL = ( 'ANY' => bless ['type' => 255], __PACKAGE__ );
+our %_LOADED = %_MINIMAL;
 
 sub _subclass {
 	my $class   = shift;
@@ -686,30 +676,28 @@ sub _subclass {
 	my $default = shift;
 
 	unless ( $_LOADED{$rrname} ) {
-		my $number = typebyname($rrname);
-		my $rrtype = "TYPE$number";
+		local @INC = LIB;
+		my $rrtype = typebyname($rrname);
 
 		unless ( $_LOADED{$rrtype} ) {			# load once only
-			my $mnemon = typebyval($number);
+			my $mnemon = typebyval($rrtype);
 			$mnemon =~ s/[^A-Za-z0-9]//g;		# expect the unexpected
 
-			my $module = join '::', __PACKAGE__, $mnemon;
+			my $subclass = join '::', __PACKAGE__, $mnemon;
 
-			unless ( eval "require $module" ) {
-				local @INC = @INC;
-				my @spec = Net::DNS::Parameters::_typespec("$number.RRTYPE");
-				if (DNSEXTLANG) {
-					my $pipe = new FileHandle("RRTYPEgen @spec |");
-					push @INC, eval "sub {$pipe}" if scalar @spec;
-				}
-				$module = join '::', __PACKAGE__, $rrtype;
-				eval "require $module";
+			unless ( eval "require $subclass" ) {
+				push @INC, sub {
+					Net::DNS::Parameters::_typespec("$rrtype.RRTYPE");
+				};
+
+				$subclass = join '::', __PACKAGE__, "TYPE$rrtype";
+				eval "require $subclass";
 			}
 
-			# cache pre-built minimal and populated default object images
-			my $subclass = $@ ? __PACKAGE__ : $module;
-			my @base = ( 'type' => $number, $@ ? ( 'module' => $module ) : () );
+			$subclass = __PACKAGE__ if $@;
 
+			# cache pre-built minimal and populated default object images
+			my @base = ( 'type' => $rrtype );
 			$_MINIMAL{$rrtype} = bless [@base], $subclass;
 
 			my $object = bless {@base}, $subclass;
@@ -755,7 +743,7 @@ sub _wrap {
 
 ################################################################################
 
-use vars qw($AUTOLOAD);
+our $AUTOLOAD;
 
 sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
@@ -770,7 +758,8 @@ sub AUTOLOAD {				## Default method
 
 	my $string = $self->string;
 	my @object = grep defined($_), $oref, $oref->VERSION;
-	eval("require $self->{module}") if $oref eq __PACKAGE__;
+	my $module = join '::', __PACKAGE__, typebyval( $self->{type} );
+	eval("require $module") if $oref eq __PACKAGE__;
 
 	@_ = (<<"END");
 ***  FATAL PROGRAM ERROR!!	Unknown method '$method'
