@@ -1,13 +1,13 @@
 package Net::DNS::RR::SPF;
 
 #
-# $Id: SPF.pm 1339 2015-04-16 13:01:17Z willem $
+# $Id: SPF.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1339 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR::TXT);
 
 =head1 NAME
@@ -21,8 +21,8 @@ use integer;
 
 
 sub spfdata {
-	return shift->char_str_list(@_) if wantarray;
-	join '', shift->char_str_list(@_);
+	my @spf = shift->char_str_list(@_);
+	wantarray ? @spf : join '', @spf;
 }
 
 sub txtdata { &spfdata; }
@@ -66,6 +66,8 @@ other unpredictable behaviour.
 
 =head2 spfdata
 
+=head2 txtdata
+
     $string = $rr->spfdata;
     @list   = $rr->spfdata;
 
