@@ -1,30 +1,11 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1489 2016-06-22 07:45:40Z willem $
+# $Id: Base.pm 1527 2017-01-18 21:42:48Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1489 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
 
 
-# Allow taint tests to be optimised away when appropriate.
-use constant UTIL  => defined eval 'require Scalar::Util';
-use constant UNCND => $] < 5.008;	## eval '${^TAINT}' breaks old compilers
-use constant TAINT => UTIL && ( UNCND || eval '${^TAINT}' );
-
-
-use strict;
-use integer;
-use Carp;
-use IO::Select;
-use IO::Socket;
-
-use Net::DNS::RR;
-use Net::DNS::Packet;
-
-use constant INT16SZ  => 2;
-use constant PACKETSZ => 512;
-
 #
 #  Implementation notes wrt IPv6 support when using perl before 5.20.0.
 #
@@ -58,6 +39,26 @@ use constant IPv6 => USE_SOCKET_IP || USE_SOCKET_INET6;
 use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 
 
+# Allow taint tests to be optimised away when appropriate.
+use constant UTIL  => defined eval 'require Scalar::Util';
+use constant UNCND => $] < 5.008;	## eval '${^TAINT}' breaks old compilers
+use constant TAINT => UTIL && ( UNCND || eval '${^TAINT}' );
+
+
+use strict;
+use warnings;
+use integer;
+use Carp;
+use IO::Select;
+use IO::Socket;
+
+use Net::DNS::RR;
+use Net::DNS::Packet;
+
+use constant INT16SZ  => 2;
+use constant PACKETSZ => 512;
+
+
 #
 # Set up a closure to be our class data.
 #
@@ -78,7 +79,6 @@ use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 		defnames	=> 1,
 		dnsrch		=> 1,
 		debug		=> 0,
-		tsig_rr		=> undef,
 		tcp_timeout	=> 120,
 		udp_timeout	=> 30,
 		persistent_tcp	=> ( SOCKS ? 1 : 0 ),
@@ -99,34 +99,10 @@ use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 
 
 # These are the attributes that the user may specify in the new() constructor.
-my %public_attr = map { $_ => $_ } qw(
-		nameserver
-		nameservers
-		port
-		srcaddr
-		srcport
-		domain
-		searchlist
-		retrans
-		retry
-		usevc
-		igntc
-		recurse
-		defnames
-		dnsrch
-		debug
-		tcp_timeout
-		udp_timeout
-		persistent_tcp
-		persistent_udp
-		dnssec
-		adflag
-		cdflag
-		force_v4
-		force_v6
-		prefer_v4
-		prefer_v6
-		);
+my %public_attr = (
+	map( {$_ => $_} keys %{&_defaults}, qw(domain nameserver nameservers prefer_v6 srcaddr) ),
+	map( {$_ => 0} qw(nameserver4 nameserver6 srcaddr4 srcaddr6) ),
+	);
 
 
 my $initial;
@@ -446,6 +422,7 @@ sub _send_tcp {
 	my $tcp_packet = pack 'n a*', length($packet_data), $packet_data;
 	my @ns = $self->nameservers();
 	my $lastanswer;
+	my $timeout = $self->{tcp_timeout};
 
 	foreach my $ip (@ns) {
 		my $socket = $self->_create_tcp_socket($ip) || next;
@@ -456,35 +433,31 @@ sub _send_tcp {
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
 
-		next unless $select->can_read( $self->{tcp_timeout} );
+		next unless $select->can_read($timeout);	# uncoverable branch
 
 		my $buffer = _read_tcp($socket);
 		$self->answerfrom($ip);
 		$self->_diag( 'answer from', "[$ip]", length($buffer), 'bytes' );
 
-		my $ans = Net::DNS::Packet->new( \$buffer, $self->{debug} );
-		$self->errorstring($@);
-
-		next unless $ans;
-
-		if ( $self->{tsig_rr} && !$ans->verify($packet) ) {
-			$self->errorstring( $ans->verifyerr );
-			return;
-		}
+		my $ans = $self->_decode_reply( \$buffer, $packet ) || next;
 
 		$ans->answerfrom($ip);
+		$lastanswer = $ans;
 
 		my $rcode = $ans->header->rcode;
-		$self->errorstring($rcode);			# historical quirk
-		return $ans if $rcode eq "NOERROR";
-		return $ans if $rcode eq "NXDOMAIN";
-
-		$self->_diag("RCODE: $rcode; try next nameserver");
-		$lastanswer = $ans;
+		last if $rcode eq 'NOERROR';
+		last if $rcode eq 'NXDOMAIN';
+		$self->errorstring($rcode);
 	}
 
-	$self->errorstring( $lastanswer->header->rcode ) if $lastanswer;
-	$self->_diag( $self->errorstring );
+	return unless $lastanswer;
+
+	if ( $self->{tsig_rr} && !$lastanswer->verify($packet) ) {
+		$self->_diag( $self->errorstring( $lastanswer->verifyerr ) );
+		return;
+	}
+
+	$self->errorstring( $lastanswer->header->rcode );	# historical quirk
 	return $lastanswer;
 }
 
@@ -539,40 +512,35 @@ NAMESERVER: foreach my $ns (@ns) {
 				my $buffer = _read_udp( $socket, $self->_packetsz );
 				$self->_diag( "answer from [$peer]", length($buffer), 'bytes' );
 
-				my $ans = Net::DNS::Packet->new( \$buffer, $self->{debug} );
-				$self->errorstring($@);
-
-				next unless $ans;
-
-				my $header = $ans->header;
-				next unless $header->qr;
-				next unless $header->id == $packet->header->id;
-
-				if ( $self->{tsig_rr} && !$ans->verify($packet) ) {
-					$self->errorstring( $ans->verifyerr );
-					return;
-				}
+				my $ans = $self->_decode_reply( \$buffer, $packet ) || next;
 
 				$ans->answerfrom($peer);
-
-				my $rcode = $header->rcode;
-				$self->errorstring($rcode);	# historical quirk
-				return $ans if $rcode eq "NOERROR";
-				return $ans if $rcode eq "NXDOMAIN";
-
-				my $msg = $$ns[3] = "RCODE: $rcode";
-				$self->_diag("$msg; try next nameserver");
 				$lastanswer = $ans;
+
+				my $rcode = $ans->header->rcode;
+				last if $rcode eq 'NOERROR';
+				last if $rcode eq 'NXDOMAIN';
+
+				$self->errorstring( $$ns[3] = $rcode );
 			}					#SELECTOR LOOP
+
+			next unless $lastanswer;
+
+			if ( $self->{tsig_rr} && !$lastanswer->verify($packet) ) {
+				my $error = $$ns[3] = $lastanswer->verifyerr;
+				$self->_diag( $self->errorstring($error) );
+				next;
+			}
+
+			$self->errorstring( $lastanswer->header->rcode );    # historical quirk
+			return $lastanswer;
 		}						#NAMESERVER LOOP
 		no integer;
 		$timeout += $timeout;
 	}							#RETRY LOOP
 
-	my $error = scalar( $select->handles ) ? 'query timed out' : $self->errorstring;
-	$error = $lastanswer->header->rcode if $lastanswer;
-	$self->_diag( $self->errorstring($error) );
-	return $lastanswer;
+	$self->_diag( $self->errorstring('query timed out') ) unless $lastanswer;
+	return;
 }
 
 
@@ -600,6 +568,7 @@ sub _bgsend_tcp {
 
 		$self->_diag( 'bgsend', "[$ip]" );
 
+		$socket->blocking(0);
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
 
@@ -698,20 +667,26 @@ sub _bgread {
 	my $buffer = $dgram ? _read_udp( $handle, $self->_packetsz ) : _read_tcp($handle);
 	$self->_diag( "answer from [$peer]", length($buffer), 'bytes' );
 
-	my $ans = Net::DNS::Packet->new( \$buffer, $self->{debug} );
+	my $reply = $self->_decode_reply( \$buffer, $query ) || return;
+
+	return $reply unless $self->{tsig_rr} && !$reply->verify($query);
+	$self->errorstring( $reply->verifyerr );
+	return;
+}
+
+
+sub _decode_reply {
+	my ( $self, $bufref, $query ) = @_;
+
+	my $reply = Net::DNS::Packet->decode( $bufref, $self->{debug} );
 	$self->errorstring($@);
-	return unless $ans;
 
-	my $header = $ans->header;
+	return unless $reply;
+
+	my $header = $reply->header;
 	return unless $header->qr;
-
-	if ( $self->{tsig_rr} && !$ans->verify($query) ) {
-		$self->errorstring( $ans->verifyerr );
-		return;
-	}
-
-	$ans->answerfrom($peer);
-	return ( ref($query) && ( $header->id != $query->header->id ) ) ? undef : $ans;
+	return unless $query;
+	return ( $header->id != $query->header->id ) ? undef : $reply;
 }
 
 
@@ -719,14 +694,14 @@ sub axfr {				## zone transfer
 	eval {
 		my $self = shift;
 
-		my ( $verify, @rr, $soa ) = $self->_axfr_start(@_);    # iterator state
+		my ( $select, $verify, @rr, $soa ) = $self->_axfr_start(@_);
 
 		my $iterator = sub {	## iterate over RRs
 			my $rr = shift(@rr);
 
 			if ( ref($rr) eq 'Net::DNS::RR::SOA' ) {
 				return $soa = $rr unless $soa;
-				$self->{axfr_sel} = undef;
+				$select = undef;
 				return if $rr->encode eq $soa->encode;
 				croak $self->errorstring('mismatched final SOA');
 			}
@@ -734,14 +709,11 @@ sub axfr {				## zone transfer
 			return $rr if scalar @rr;
 
 			my $reply;
-			( $reply, $verify ) = $self->_axfr_next($verify);
-			return $self->{axfr_sel} = undef unless $reply;
+			( $reply, $verify ) = $self->_axfr_next( $select, $verify );
 			@rr = $reply->answer;
 			return $rr;
 		};
 
-		$iterator->();					# read initial packet
-
 		return $iterator unless wantarray;
 
 		my @zone;					# assemble whole zone
@@ -776,47 +748,58 @@ sub _axfr_start {
 
 	$self->_diag("axfr_start( $dname @class )");
 
+	my ( $select, $reply, $rcode );
 	foreach my $ns ( $self->nameservers ) {
 		my $socket = $self->_create_tcp_socket($ns) || next;
 
 		$self->_diag("axfr_start nameserver [$ns]");
 
-		$self->{axfr_sel} = IO::Select->new($socket);
+		$select = IO::Select->new($socket);
 		$socket->send($TCP_msg);
 		$self->errorstring($!);
 
-		return $request->sigrr ? $request : undef;
+		($reply) = $self->_axfr_next($select);
+		last if ( $rcode = $reply->header->rcode ) eq 'NOERROR';
 	}
 
-	croak $self->errorstring;
+	croak $self->errorstring unless $reply;
+
+	$self->errorstring($rcode);				# historical quirk
+
+	my $verify = $request->sigrr ? $request : undef;
+	unless ($verify) {
+		croak $self->errorstring unless $rcode eq 'NOERROR';
+		return ( $select, $verify, $reply->answer );
+	}
+
+	my $verifyok = $reply->verify($verify);
+	croak $self->errorstring( $reply->verifyerr ) unless $verifyok;
+	croak $self->errorstring unless $rcode eq 'NOERROR';
+	return ( $select, $verifyok, $reply->answer );
 }
 
 
 sub _axfr_next {
-	my ( $self, $verify ) = @_;
+	my $self   = shift;
+	my $select = shift || return;
+	my $verify = shift;
 
-	my $select = $self->{axfr_sel} || return;
-	my ($sock) = $select->can_read( $self->{tcp_timeout} );
-	croak $self->errorstring('timed out') unless $sock;
+	my ($socket) = $select->can_read( $self->{tcp_timeout} );
+	croak $self->errorstring('timed out') unless $socket;
 
-	my $peerhost = $sock->peerhost;
-	$self->answerfrom($peerhost);
+	$self->answerfrom( $socket->peerhost );
 
-	my $buffer = _read_tcp($sock);
+	my $buffer = _read_tcp($socket);
 	$self->_diag( 'received', length($buffer), 'bytes' );
 
 	my $packet = Net::DNS::Packet->new( \$buffer );
 	croak $@, $self->errorstring('corrupt packet') if $@;
 
-	if ( $verify && not( $verify = $packet->verify($verify) ) ) {
-		croak $self->errorstring( $packet->verifyerr );
-	}
+	return ( $packet, $verify ) unless $verify;
 
-	my $rcode = $packet->header->rcode;
-	croak $self->errorstring($rcode) unless $rcode eq 'NOERROR';
-
-	$packet->answerfrom($peerhost);
-	return ( $packet, $verify );
+	my $verifyok = $packet->verify($verify);
+	croak $self->errorstring( $packet->verifyerr ) unless $verifyok;
+	return ( $packet, $verifyok );
 }
 
 
@@ -831,7 +814,7 @@ sub _read_tcp {
 	my ($unread) = unpack 'n*', $size_buf;
 
 	my $buffer = '';
-	while ($unread) {
+	while ( $unread > 0 ) {
 
 		# During some of my tests recv() returned undef even
 		# though there was no error.  Checking for the amount
@@ -840,14 +823,9 @@ sub _read_tcp {
 		my $read_buf = '';
 		$socket->recv( $read_buf, $unread );
 
-		my $read = length $read_buf;
-		last unless $read;
-
+		$unread -= length( $read_buf || last );
 		$buffer .= $read_buf;
-		$unread -= $read;
 	}
-
-	warn "ERROR: tcp recv failed: $!\n" if $unread;
 	return $buffer;
 }
 
@@ -878,7 +856,7 @@ sub _create_tcp_socket {
 
 	my $dstport = $self->{port};
 	my $srcport = $self->{srcport};
-	my $timeout = $self->{tcp_timeout} || 1;
+	my $timeout = $self->{tcp_timeout};
 
 	if (USE_SOCKET_IP) {
 		my $srcaddr = _ipv6($ip) ? $self->{srcaddr6} : $self->{srcaddr4};
@@ -1020,7 +998,7 @@ sub _make_query_packet {
 		my $header = $packet->header;
 		$header->ad( $self->{adflag} );			# RFC6840, 5.7
 		$header->cd( $self->{cdflag} );			# RFC6840, 5.9
-		$header->do(1) if $self->dnssec;
+		$header->do(1) if $self->{dnssec};
 		$header->rd( $self->{recurse} );
 	}
 
@@ -1117,7 +1095,10 @@ sub udppacketsize {
 my $warned;
 
 sub make_query_packet {			## historical
-	carp 'deprecated method; see RT#37104' unless $warned++;    # uncoverable pod
+	unless ( $warned++ ) {					# uncoverable pod
+		local $SIG{__WARN__};
+		carp 'deprecated method; see RT#37104';
+	}
 	&_make_query_packet;
 }
 
@@ -1128,7 +1109,7 @@ sub _diag {				## debug output
 }
 
 
-use vars qw($AUTOLOAD);
+our $AUTOLOAD;
 
 sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
